Class: Decidim::Initiatives::CreateInitiative

Inherits:
Command
  • Object
show all
Includes:
CurrentLocale, MultipleAttachmentsMethods
Defined in:
app/commands/decidim/initiatives/create_initiative.rb

Overview

A command with all the business logic that creates a new initiative.

Instance Method Summary collapse

Methods included from CurrentLocale

#current_locale

Constructor Details

#initialize(form) ⇒ CreateInitiative

Public: Initializes the command.

form - A form object with the params.



16
17
18
# File 'app/commands/decidim/initiatives/create_initiative.rb', line 16

def initialize(form)
  @form = form
end

Instance Method Details

#callObject

Executes the command. Broadcasts these events:

  • :ok when everything is valid.
  • :invalid if the form was not valid and we could not proceed.

Returns nothing.



26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
# File 'app/commands/decidim/initiatives/create_initiative.rb', line 26

def call
  return broadcast(:invalid) if form.invalid?

  if process_attachments?
    build_attachments
    return broadcast(:invalid) if attachments_invalid?
  end

  initiative = create_initiative

  if initiative.persisted?
    broadcast(:ok, initiative)
  else
    broadcast(:invalid, initiative)
  end
end