Class: Decidim::Initiative

Inherits:
ApplicationRecord
  • Object
show all
Includes:
ActiveModel::Dirty, Authorable, Comments::Commentable, DownloadYourData, FilterableResource, Followable, HasArea, HasAttachmentCollections, HasAttachments, HasReference, HasResourcePermission, Decidim::Initiatives::HasArea, Decidim::Initiatives::InitiativeSlug, Loggable, Participable, Publicable, Randomable, Reportable, Resourceable, ScopableParticipatorySpace, Searchable, ShareableWithToken, Traceable, TranslatableResource
Defined in:
app/models/decidim/initiative.rb

Overview

The data store for a Initiative in the Decidim::Initiatives component.

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Decidim::Initiatives::InitiativeSlug

#id_from_slug, #slug_from_id

Class Method Details

.export_serializerObject



161
162
163
# File 'app/models/decidim/initiative.rb', line 161

def self.export_serializer
  Decidim::Initiatives::DownloadYourDataInitiativeSerializer
end

.log_presenter_class_for(_log) ⇒ Object



165
166
167
# File 'app/models/decidim/initiative.rb', line 165

def self.log_presenter_class_for(_log)
  Decidim::Initiatives::AdminLog::InitiativePresenter
end

.ransack(params = {}, options = {}) ⇒ Object



476
477
478
# File 'app/models/decidim/initiative.rb', line 476

def self.ransack(params = {}, options = {})
  Initiatives::InitiativeSearch.new(self, params, options)
end

.ransackable_associations(_auth_object = nil) ⇒ Object



177
178
179
# File 'app/models/decidim/initiative.rb', line 177

def self.ransackable_associations(_auth_object = nil)
  %w(area scope taxonomies)
end

.ransackable_attributes(auth_object = nil) ⇒ Object



169
170
171
172
173
174
175
# File 'app/models/decidim/initiative.rb', line 169

def self.ransackable_attributes(auth_object = nil)
  base = %w(search_text title description id id_string supports_count author_name author_nickname)

  return base unless auth_object&.admin?

  base + %w(published_at created_at state decidim_area_id type_id)
end

.ransackable_scopes(_auth_object = nil) ⇒ Object



181
182
183
# File 'app/models/decidim/initiative.rb', line 181

def self.ransackable_scopes(_auth_object = nil)
  [:with_any_state, :with_any_type, :with_any_scope, :with_any_area]
end

Instance Method Details

#accepts_offline_votes?Boolean

Returns:

  • (Boolean)


419
420
421
# File 'app/models/decidim/initiative.rb', line 419

def accepts_offline_votes?
  published? && (offline_signature_type? || any_signature_type?)
end

#accepts_online_unvotes?Boolean

Returns:

  • (Boolean)


427
428
429
# File 'app/models/decidim/initiative.rb', line 427

def accepts_online_unvotes?
  accepts_online_votes? && type.undo_online_signatures_enabled?
end

#accepts_online_votes?Boolean

Returns:

  • (Boolean)


423
424
425
# File 'app/models/decidim/initiative.rb', line 423

def accepts_online_votes?
  votes_enabled? && (online_signature_type? || any_signature_type?)
end

#allow_resource_permissions?Boolean

Public: Overrides the ‘allow_resource_permissions?` Resourceable concern method.

Returns:

  • (Boolean)


464
465
466
# File 'app/models/decidim/initiative.rb', line 464

def allow_resource_permissions?
  true
end

#answered?Boolean

Public: Checks if the organization has given an answer for the initiative.

Returns a Boolean.

Returns:

  • (Boolean)


235
236
237
# File 'app/models/decidim/initiative.rb', line 235

def answered?
  answered_at.present?
end

#author_nameObject

Public: Returns the author name. If it has been created by an organization it will return the organization’s name. Otherwise it will return author’s name.

Returns a string



210
211
212
# File 'app/models/decidim/initiative.rb', line 210

def author_name
  user_group&.name || author.name
end

#author_usersObject



415
416
417
# File 'app/models/decidim/initiative.rb', line 415

def author_users
  [author].concat(committee_members.excluding_author.map(&:user))
end

Public: Overrides participatory space’s banner image with the banner image defined for the initiative type.

Returns Decidim::BannerImageUploader



189
190
191
# File 'app/models/decidim/initiative.rb', line 189

def banner_image
  type.attached_uploader(:banner_image)
end

#commentable?Boolean

Public: Whether the object’s comments are visible or not.

Returns:

  • (Boolean)


194
195
196
# File 'app/models/decidim/initiative.rb', line 194

def commentable?
  type.comments_enabled?
end

#comments_have_alignment?Boolean

Public: Overrides the ‘comments_have_alignment?` Commentable concern method.

Returns:

  • (Boolean)


396
397
398
# File 'app/models/decidim/initiative.rb', line 396

def comments_have_alignment?
  true
end

#comments_have_votes?Boolean

Public: Overrides the ‘comments_have_votes?` Commentable concern method.

Returns:

  • (Boolean)


401
402
403
# File 'app/models/decidim/initiative.rb', line 401

def comments_have_votes?
  true
end

#componentObject



443
444
445
# File 'app/models/decidim/initiative.rb', line 443

def component
  nil
end

#created_by_individual?Boolean

Public: Check if an initiative has been created by an individual person. If it is false, then it has been created by an authorized organization.

Returns a Boolean

Returns:

  • (Boolean)


202
203
204
# File 'app/models/decidim/initiative.rb', line 202

def created_by_individual?
  decidim_user_group_id.nil?
end

#enough_committee_members?Boolean

Returns:

  • (Boolean)


435
436
437
# File 'app/models/decidim/initiative.rb', line 435

def enough_committee_members?
  committee_members.approved.count >= minimum_committee_members
end

#has_authorship?(user) ⇒ Boolean

Public: Checks if user is the author or is part of the promotal committee of the initiative.

Returns a Boolean.

Returns:

  • (Boolean)


409
410
411
412
413
# File 'app/models/decidim/initiative.rb', line 409

def has_authorship?(user)
  return true if author.id == user.id

  committee_members.approved.where(decidim_users_id: user.id).any?
end

#has_signature_interval_defined?Boolean

Public: Returns whether the signature interval is already defined or not.

Returns:

  • (Boolean)


277
278
279
# File 'app/models/decidim/initiative.rb', line 277

def has_signature_interval_defined?
  signature_end_date.present? && signature_start_date.present?
end

#hashtagObject

Public: Returns the hashtag for the initiative.



282
283
284
# File 'app/models/decidim/initiative.rb', line 282

def hashtag
  attributes["hashtag"].to_s.delete("#")
end

#minimum_committee_membersObject



431
432
433
# File 'app/models/decidim/initiative.rb', line 431

def minimum_committee_members
  type.minimum_committee_members || Decidim::Initiatives.minimum_committee_members
end

#missing_committee_membersObject



439
440
441
# File 'app/models/decidim/initiative.rb', line 439

def missing_committee_members
  minimum_committee_members - committee_members.approved.count
end

#offline_votes_countObject



331
332
333
334
335
# File 'app/models/decidim/initiative.rb', line 331

def offline_votes_count
  return 0 if online_signature_type?

  offline_votes["total"].to_i
end

#offline_votes_count_for(scope) ⇒ Object



345
346
347
348
349
350
351
# File 'app/models/decidim/initiative.rb', line 345

def offline_votes_count_for(scope)
  return 0 if online_signature_type?

  scope_key = (scope&.id || "global").to_s

  (offline_votes || {}).fetch(scope_key, 0).to_i
end

#online_votes_countObject

Public: Calculates all the votes across all the scopes.

Returns an Integer.



325
326
327
328
329
# File 'app/models/decidim/initiative.rb', line 325

def online_votes_count
  return 0 if offline_signature_type?

  online_votes["total"].to_i
end

#online_votes_count_for(scope) ⇒ Object



337
338
339
340
341
342
343
# File 'app/models/decidim/initiative.rb', line 337

def online_votes_count_for(scope)
  return 0 if offline_signature_type?

  scope_key = (scope&.id || "global").to_s

  (online_votes || {}).fetch(scope_key, 0).to_i
end

#percentageObject

Public: Returns the percentage of required supports reached



308
309
310
# File 'app/models/decidim/initiative.rb', line 308

def percentage
  [supports_count * 100 / supports_required, 100].min
end

#publish!Object

Public: Publishes this initiative

Returns true if the record was properly saved, false otherwise.



256
257
258
259
260
261
262
263
264
265
# File 'app/models/decidim/initiative.rb', line 256

def publish!
  return false if published?

  update(
    published_at: Time.current,
    state: "open",
    signature_start_date: Date.current,
    signature_end_date: signature_end_date || (Date.current + Decidim::Initiatives.default_signature_time_period_length)
  )
end

#reported_attributesObject

Public: Overrides the ‘reported_attributes` Reportable concern method.



215
216
217
# File 'app/models/decidim/initiative.rb', line 215

def reported_attributes
  [:title, :description]
end

#scopes_enabledObject

Public: Overrides scopes enabled attribute value. For initiatives it will not be directly managed by the user and it will be enabled by default.



249
250
251
# File 'app/models/decidim/initiative.rb', line 249

def scopes_enabled
  true
end

#scopes_enabled?Boolean

Public: Overrides scopes enabled flag available in other models like participatory space or assemblies. For initiatives it will not be directly managed by the user and it will be enabled by default.

Returns:

  • (Boolean)


242
243
244
# File 'app/models/decidim/initiative.rb', line 242

def scopes_enabled?
  true
end

#set_offline_votes_totalObject



365
366
367
368
369
# File 'app/models/decidim/initiative.rb', line 365

def set_offline_votes_total
  return if offline_votes.blank?

  offline_votes["total"] = offline_votes[scope&.id.to_s] || offline_votes["global"]
end

#shareable_url(share_token) ⇒ Object



472
473
474
# File 'app/models/decidim/initiative.rb', line 472

def shareable_url(share_token)
  EngineRouter.main_proxy(self).initiative_url(self, share_token: share_token.token)
end

#slugObject

Public: Overrides slug attribute from participatory processes.



386
387
388
# File 'app/models/decidim/initiative.rb', line 386

def slug
  slug_from_id(id)
end

#supports_countObject

Public: Calculates the number of total current supports.

Returns an Integer.



289
290
291
# File 'app/models/decidim/initiative.rb', line 289

def supports_count
  online_votes_count + offline_votes_count
end

#supports_count_for(scope) ⇒ Object

Public: Calculates the number of current supports for a scope.

Returns an Integer.



296
297
298
# File 'app/models/decidim/initiative.rb', line 296

def supports_count_for(scope)
  online_votes_count_for(scope) + offline_votes_count_for(scope)
end

#supports_goal_reached?Boolean

Public: Whether the supports required objective has been reached

Returns:

  • (Boolean)


313
314
315
# File 'app/models/decidim/initiative.rb', line 313

def supports_goal_reached?
  votable_initiative_type_scopes.map(&:scope).all? { |scope| supports_goal_reached_for?(scope) }
end

#supports_goal_reached_for?(scope) ⇒ Boolean

Public: Whether the supports required objective has been reached for a scope

Returns:

  • (Boolean)


318
319
320
# File 'app/models/decidim/initiative.rb', line 318

def supports_goal_reached_for?(scope)
  supports_count_for(scope) >= supports_required_for(scope)
end

#supports_required_for(scope) ⇒ Object

Public: Calculates the number of supports required to accept the initiative for a scope.

Returns an Integer.



303
304
305
# File 'app/models/decidim/initiative.rb', line 303

def supports_required_for(scope)
  initiative_type_scopes.find_by(decidim_scopes_id: scope&.id).supports_required
end

#to_paramObject



390
391
392
# File 'app/models/decidim/initiative.rb', line 390

def to_param
  slug
end

#unpublish!Object

Public: Unpublishes this initiative

Returns true if the record was properly saved, false otherwise.



270
271
272
273
274
# File 'app/models/decidim/initiative.rb', line 270

def unpublish!
  return false unless published?

  update(published_at: nil, state: "discarded")
end

#update_online_votes_countersObject



353
354
355
356
357
358
359
360
361
362
363
# File 'app/models/decidim/initiative.rb', line 353

def update_online_votes_counters
  online_votes = votes.group(:scope).count.each_with_object({}) do |(scope, count), counters|
    counters[scope&.id || "global"] = count
    counters["total"] ||= 0
    counters["total"] += count
  end

  # rubocop:disable Rails/SkipsModelValidations
  update_column("online_votes", online_votes)
  # rubocop:enable Rails/SkipsModelValidations
end

#user_allowed_to_comment?(user) ⇒ Boolean

Returns:

  • (Boolean)


468
469
470
# File 'app/models/decidim/initiative.rb', line 468

def user_allowed_to_comment?(user)
  ActionAuthorizer.new(user, "comment", self, nil).authorize.ok?
end

#user_role_config_for(_user, _role_name) ⇒ Object

Public: Returns an empty object. This method should be implemented by ‘ParticipatorySpaceResourceable`, but for some reason this model does not implement this interface.



459
460
461
# File 'app/models/decidim/initiative.rb', line 459

def user_role_config_for(_user, _role_name)
  Decidim::ParticipatorySpaceRoleConfig::Base.new(:empty_role_name)
end

#validate_sms_code_on_votes?Boolean

Public: Checks if the type the initiative belongs to enables SMS code verification step. Tis configuration is ignored if the organization does not have the sms authorization available

Returns a Boolean

Returns:

  • (Boolean)


452
453
454
# File 'app/models/decidim/initiative.rb', line 452

def validate_sms_code_on_votes?
  organization.available_authorizations.include?("sms") && type.validate_sms_code_on_votes?
end

#votable_initiative_type_scopesObject

Public: Finds all the InitiativeTypeScopes that are eligible to be voted by a user. Usually this is only the ‘scoped_type` but voting on children of the scoped type is enabled we have to filter all the available scopes in the initiative type to select the ones that are a descendant of the initiative type.

Returns an Array of Decidim::InitiativesScopeType.



377
378
379
380
381
382
383
# File 'app/models/decidim/initiative.rb', line 377

def votable_initiative_type_scopes
  return Array(scoped_type) unless type.child_scope_threshold_enabled?

  initiative_type_scopes.select do |initiative_type_scope|
    initiative_type_scope.scope.present? && (scoped_type.global_scope? || scoped_type.scope.ancestor_of?(initiative_type_scope.scope))
  end.prepend(scoped_type).uniq
end

#voted_by?(user) ⇒ Boolean

Public: Check if the user has voted the question.

Returns Boolean.

Returns:

  • (Boolean)


228
229
230
# File 'app/models/decidim/initiative.rb', line 228

def voted_by?(user)
  votes.where(author: user).any?
end

#votes_enabled?Boolean

Returns:

  • (Boolean)


219
220
221
222
223
# File 'app/models/decidim/initiative.rb', line 219

def votes_enabled?
  published? &&
    signature_start_date <= Date.current &&
    signature_end_date >= Date.current
end