Class: Decidim::Initiative

Inherits:
ApplicationRecord
  • Object
show all
Includes:
ActiveModel::Dirty, Authorable, Comments::Commentable, FilterableResource, Followable, HasArea, HasAttachmentCollections, HasAttachments, HasReference, HasResourcePermission, Decidim::Initiatives::HasArea, Decidim::Initiatives::InitiativeSlug, Loggable, Participable, Publicable, Randomable, Reportable, Resourceable, ScopableParticipatorySpace, Searchable, Traceable, TranslatableResource
Defined in:
app/models/decidim/initiative.rb

Overview

The data store for a Initiative in the Decidim::Initiatives component.

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Decidim::Initiatives::InitiativeSlug

#id_from_slug, #slug_from_id

Class Method Details

.log_presenter_class_for(_log) ⇒ Object



159
160
161
# File 'app/models/decidim/initiative.rb', line 159

def self.log_presenter_class_for(_log)
  Decidim::Initiatives::AdminLog::InitiativePresenter
end

.ransack(params = {}, options = {}) ⇒ Object



474
475
476
# File 'app/models/decidim/initiative.rb', line 474

def self.ransack(params = {}, options = {})
  Initiatives::InitiativeSearch.new(self, params, options)
end

.ransackable_scopes(_auth_object = nil) ⇒ Object



163
164
165
# File 'app/models/decidim/initiative.rb', line 163

def self.ransackable_scopes(_auth_object = nil)
  [:with_any_state, :with_any_type, :with_any_scope, :with_any_area]
end

Instance Method Details

#accepts_offline_votes?Boolean

Returns:

  • (Boolean)


421
422
423
# File 'app/models/decidim/initiative.rb', line 421

def accepts_offline_votes?
  published? && (offline_signature_type? || any_signature_type?)
end

#accepts_online_unvotes?Boolean

Returns:

  • (Boolean)


429
430
431
# File 'app/models/decidim/initiative.rb', line 429

def accepts_online_unvotes?
  accepts_online_votes? && type.undo_online_signatures_enabled?
end

#accepts_online_votes?Boolean

Returns:

  • (Boolean)


425
426
427
# File 'app/models/decidim/initiative.rb', line 425

def accepts_online_votes?
  votes_enabled? && (online_signature_type? || any_signature_type?)
end

#allow_resource_permissions?Boolean

Public: Overrides the ‘allow_resource_permissions?` Resourceable concern method.

Returns:

  • (Boolean)


466
467
468
# File 'app/models/decidim/initiative.rb', line 466

def allow_resource_permissions?
  true
end

#answered?Boolean

Public: Checks if the organization has given an answer for the initiative.

Returns a Boolean.

Returns:

  • (Boolean)


237
238
239
# File 'app/models/decidim/initiative.rb', line 237

def answered?
  answered_at.present?
end

#author_nameObject

Public: Returns the author name. If it has been created by an organization it will return the organization’s name. Otherwise it will return author’s name.

Returns a string



212
213
214
# File 'app/models/decidim/initiative.rb', line 212

def author_name
  user_group&.name || author.name
end

#author_usersObject



417
418
419
# File 'app/models/decidim/initiative.rb', line 417

def author_users
  [author].concat(committee_members.excluding_author.map(&:user))
end

Public: Overrides participatory space’s banner image with the banner image defined for the initiative type.

Returns Decidim::BannerImageUploader



171
172
173
# File 'app/models/decidim/initiative.rb', line 171

def banner_image
  type.attached_uploader(:banner_image)
end

#closed?Boolean

Public: Checks if an initiative is closed. An initiative is closed when at least one of the following conditions is true:

  • It has been discarded.

  • It has been rejected.

  • It has been accepted.

  • Signature collection period has finished.

Returns a Boolean

Returns:

  • (Boolean)


204
205
206
# File 'app/models/decidim/initiative.rb', line 204

def closed?
  discarded? || rejected? || accepted? || !votes_enabled?
end

#commentable?Boolean

Public: Whether the object’s comments are visible or not.

Returns:

  • (Boolean)


176
177
178
# File 'app/models/decidim/initiative.rb', line 176

def commentable?
  type.comments_enabled?
end

#comments_have_alignment?Boolean

Public: Overrides the ‘comments_have_alignment?` Commentable concern method.

Returns:

  • (Boolean)


398
399
400
# File 'app/models/decidim/initiative.rb', line 398

def comments_have_alignment?
  true
end

#comments_have_votes?Boolean

Public: Overrides the ‘comments_have_votes?` Commentable concern method.

Returns:

  • (Boolean)


403
404
405
# File 'app/models/decidim/initiative.rb', line 403

def comments_have_votes?
  true
end

#componentObject



445
446
447
# File 'app/models/decidim/initiative.rb', line 445

def component
  nil
end

#created_by_individual?Boolean

Public: Check if an initiative has been created by an individual person. If it is false, then it has been created by an authorized organization.

Returns a Boolean

Returns:

  • (Boolean)


184
185
186
# File 'app/models/decidim/initiative.rb', line 184

def created_by_individual?
  decidim_user_group_id.nil?
end

#enough_committee_members?Boolean

Returns:

  • (Boolean)


437
438
439
# File 'app/models/decidim/initiative.rb', line 437

def enough_committee_members?
  committee_members.approved.count >= minimum_committee_members
end

#has_authorship?(user) ⇒ Boolean

Public: Checks if user is the author or is part of the promotal committee of the initiative.

Returns a Boolean.

Returns:

  • (Boolean)


411
412
413
414
415
# File 'app/models/decidim/initiative.rb', line 411

def has_authorship?(user)
  return true if author.id == user.id

  committee_members.approved.where(decidim_users_id: user.id).any?
end

#has_signature_interval_defined?Boolean

Public: Returns wether the signature interval is already defined or not.

Returns:

  • (Boolean)


279
280
281
# File 'app/models/decidim/initiative.rb', line 279

def has_signature_interval_defined?
  signature_end_date.present? && signature_start_date.present?
end

#hashtagObject

Public: Returns the hashtag for the initiative.



284
285
286
# File 'app/models/decidim/initiative.rb', line 284

def hashtag
  attributes["hashtag"].to_s.delete("#")
end

#minimum_committee_membersObject



433
434
435
# File 'app/models/decidim/initiative.rb', line 433

def minimum_committee_members
  type.minimum_committee_members || Decidim::Initiatives.minimum_committee_members
end

#missing_committee_membersObject



441
442
443
# File 'app/models/decidim/initiative.rb', line 441

def missing_committee_members
  minimum_committee_members - committee_members.approved.count
end

#offline_votes_countObject



333
334
335
336
337
# File 'app/models/decidim/initiative.rb', line 333

def offline_votes_count
  return 0 if online_signature_type?

  offline_votes["total"].to_i
end

#offline_votes_count_for(scope) ⇒ Object



347
348
349
350
351
352
353
# File 'app/models/decidim/initiative.rb', line 347

def offline_votes_count_for(scope)
  return 0 if online_signature_type?

  scope_key = (scope&.id || "global").to_s

  (offline_votes || {}).fetch(scope_key, 0).to_i
end

#online_votes_countObject

Public: Calculates all the votes across all the scopes.

Returns an Integer.



327
328
329
330
331
# File 'app/models/decidim/initiative.rb', line 327

def online_votes_count
  return 0 if offline_signature_type?

  online_votes["total"].to_i
end

#online_votes_count_for(scope) ⇒ Object



339
340
341
342
343
344
345
# File 'app/models/decidim/initiative.rb', line 339

def online_votes_count_for(scope)
  return 0 if offline_signature_type?

  scope_key = (scope&.id || "global").to_s

  (online_votes || {}).fetch(scope_key, 0).to_i
end

#open?Boolean

Public: check if an initiative is open

Returns a Boolean

Returns:

  • (Boolean)


191
192
193
# File 'app/models/decidim/initiative.rb', line 191

def open?
  !closed?
end

#percentageObject

Public: Returns the percentage of required supports reached



310
311
312
# File 'app/models/decidim/initiative.rb', line 310

def percentage
  [supports_count * 100 / supports_required, 100].min
end

#publish!Object

Public: Publishes this initiative

Returns true if the record was properly saved, false otherwise.



258
259
260
261
262
263
264
265
266
267
# File 'app/models/decidim/initiative.rb', line 258

def publish!
  return false if published?

  update(
    published_at: Time.current,
    state: "published",
    signature_start_date: Date.current,
    signature_end_date: signature_end_date || (Date.current + Decidim::Initiatives.default_signature_time_period_length)
  )
end

#reported_attributesObject

Public: Overrides the ‘reported_attributes` Reportable concern method.



217
218
219
# File 'app/models/decidim/initiative.rb', line 217

def reported_attributes
  [:title, :description]
end

#scopes_enabledObject

Public: Overrides scopes enabled attribute value. For initiatives it will not be directly managed by the user and it will be enabled by default.



251
252
253
# File 'app/models/decidim/initiative.rb', line 251

def scopes_enabled
  true
end

#scopes_enabled?Boolean

Public: Overrides scopes enabled flag available in other models like participatory space or assemblies. For initiatives it will not be directly managed by the user and it will be enabled by default.

Returns:

  • (Boolean)


244
245
246
# File 'app/models/decidim/initiative.rb', line 244

def scopes_enabled?
  true
end

#set_offline_votes_totalObject



367
368
369
370
371
# File 'app/models/decidim/initiative.rb', line 367

def set_offline_votes_total
  return if offline_votes.blank?

  offline_votes["total"] = offline_votes[scope&.id.to_s] || offline_votes["global"]
end

#slugObject

Public: Overrides slug attribute from participatory processes.



388
389
390
# File 'app/models/decidim/initiative.rb', line 388

def slug
  slug_from_id(id)
end

#supports_countObject

Public: Calculates the number of total current supports.

Returns an Integer.



291
292
293
# File 'app/models/decidim/initiative.rb', line 291

def supports_count
  online_votes_count + offline_votes_count
end

#supports_count_for(scope) ⇒ Object

Public: Calculates the number of current supports for a scope.

Returns an Integer.



298
299
300
# File 'app/models/decidim/initiative.rb', line 298

def supports_count_for(scope)
  online_votes_count_for(scope) + offline_votes_count_for(scope)
end

#supports_goal_reached?Boolean

Public: Whether the supports required objective has been reached

Returns:

  • (Boolean)


315
316
317
# File 'app/models/decidim/initiative.rb', line 315

def supports_goal_reached?
  votable_initiative_type_scopes.map(&:scope).all? { |scope| supports_goal_reached_for?(scope) }
end

#supports_goal_reached_for?(scope) ⇒ Boolean

Public: Whether the supports required objective has been reached for a scope

Returns:

  • (Boolean)


320
321
322
# File 'app/models/decidim/initiative.rb', line 320

def supports_goal_reached_for?(scope)
  supports_count_for(scope) >= supports_required_for(scope)
end

#supports_required_for(scope) ⇒ Object

Public: Calculates the number of supports required to accept the initiative for a scope.

Returns an Integer.



305
306
307
# File 'app/models/decidim/initiative.rb', line 305

def supports_required_for(scope)
  initiative_type_scopes.find_by(decidim_scopes_id: scope&.id).supports_required
end

#to_paramObject



392
393
394
# File 'app/models/decidim/initiative.rb', line 392

def to_param
  slug
end

#unpublish!Object

Public: Unpublishes this initiative

Returns true if the record was properly saved, false otherwise.



272
273
274
275
276
# File 'app/models/decidim/initiative.rb', line 272

def unpublish!
  return false unless published?

  update(published_at: nil, state: "discarded")
end

#update_online_votes_countersObject



355
356
357
358
359
360
361
362
363
364
365
# File 'app/models/decidim/initiative.rb', line 355

def update_online_votes_counters
  online_votes = votes.group(:scope).count.each_with_object({}) do |(scope, count), counters|
    counters[scope&.id || "global"] = count
    counters["total"] ||= 0
    counters["total"] += count
  end

  # rubocop:disable Rails/SkipsModelValidations
  update_column("online_votes", online_votes)
  # rubocop:enable Rails/SkipsModelValidations
end

#user_allowed_to_comment?(user) ⇒ Boolean

Returns:

  • (Boolean)


470
471
472
# File 'app/models/decidim/initiative.rb', line 470

def user_allowed_to_comment?(user)
  ActionAuthorizer.new(user, "comment", self, nil).authorize.ok?
end

#user_role_config_for(_user, _role_name) ⇒ Object

Public: Returns an empty object. This method should be implemented by ‘ParticipatorySpaceResourceable`, but for some reason this model does not implement this interface.



461
462
463
# File 'app/models/decidim/initiative.rb', line 461

def user_role_config_for(_user, _role_name)
  Decidim::ParticipatorySpaceRoleConfig::Base.new(:empty_role_name)
end

#validate_sms_code_on_votes?Boolean

Public: Checks if the type the initiative belongs to enables SMS code verification step. Tis configuration is ignored if the organization does not have the sms authorization available

Returns a Boolean

Returns:

  • (Boolean)


454
455
456
# File 'app/models/decidim/initiative.rb', line 454

def validate_sms_code_on_votes?
  organization.available_authorizations.include?("sms") && type.validate_sms_code_on_votes?
end

#votable_initiative_type_scopesObject

Public: Finds all the InitiativeTypeScopes that are eligible to be voted by a user. Usually this is only the ‘scoped_type` but voting on children of the scoped type is enabled we have to filter all the available scopes in the initiative type to select the ones that are a descendant of the initiative type.

Returns an Array of Decidim::InitiativesScopeType.



379
380
381
382
383
384
385
# File 'app/models/decidim/initiative.rb', line 379

def votable_initiative_type_scopes
  return Array(scoped_type) unless type.child_scope_threshold_enabled?

  initiative_type_scopes.select do |initiative_type_scope|
    initiative_type_scope.scope.present? && (scoped_type.global_scope? || scoped_type.scope.ancestor_of?(initiative_type_scope.scope))
  end.prepend(scoped_type).uniq
end

#voted_by?(user) ⇒ Boolean

Public: Check if the user has voted the question.

Returns Boolean.

Returns:

  • (Boolean)


230
231
232
# File 'app/models/decidim/initiative.rb', line 230

def voted_by?(user)
  votes.where(author: user).any?
end

#votes_enabled?Boolean

Returns:

  • (Boolean)


221
222
223
224
225
# File 'app/models/decidim/initiative.rb', line 221

def votes_enabled?
  published? &&
    signature_start_date <= Date.current &&
    signature_end_date >= Date.current
end